﻿using System;
using MccDaq;

namespace AnalogInAutoRetrigger
{
    class Program
    {
        public const int BLOCKSIZE = 50;
        public const int CHANCOUNT = 4;
        public const int FIRSTCHANNEL = 0;
        public const int LASTCHANNEL = 3;
        public const int FREQ = 100;
        public const int PACKETSIZE = 32;
 
        public const string DEVICE = "USB-1208LS";

        public static MccBoard daq;

        static void Main(string[] args)
        {

            int BoardNum = 0;
            int DeviceChannels = 0;
            int Rate = FREQ;
            Console.WriteLine("\nExternal TTL trigger Example\n");
            Console.WriteLine("\nLocating board...\n");

            BoardNum = GetBoardNum(DEVICE);

            if (BoardNum == -1)
                Console.WriteLine("No {0} detected!", DEVICE);
            else
            {
                daq = new MccDaq.MccBoard(BoardNum);
                daq.BoardConfig.GetNumAdChans(out DeviceChannels);
                int chs = 0;
                daq.BoardConfig.GetNumAdChans(out chs);
                Console.WriteLine("Number of available channels {0}\n",chs);

                int BUFFERSIZE = PACKETSIZE * CHANCOUNT;
                int HALFBUFFSIZE = BUFFERSIZE / 2;
                IntPtr buffer = MccService.WinBufAllocEx(BUFFERSIZE);

                if (buffer == IntPtr.Zero)
                {
                    Console.WriteLine("Bad Handle");
                    return;
                }

                short[] chArray = new short[CHANCOUNT];
                Range[] chRange = new Range[CHANCOUNT];

                chArray[0] = 0;
                chArray[1] = 1;
                chArray[2] = 2;
                chArray[3] = 3;

                chRange[0] = Range.Bip10Volts;
                chRange[1] = Range.Bip10Volts;
                chRange[2] = Range.Bip10Volts;
                chRange[3] = Range.Bip10Volts;

                IsError( daq.ALoadQueue(chArray, chRange, CHANCOUNT));

                //TTL signal connected to Trigger input
                IsError(daq.SetTrigger(TriggerType.TrigHigh, 0, 0));
                Console.WriteLine("Waiting for level high trigger.");


                //setup the acquisiton
                IsError( daq.AInScan(   FIRSTCHANNEL,
                                        LASTCHANNEL,
                                        BUFFERSIZE,
                                        ref Rate,
                                        Range.Bip10Volts,
                                        buffer,
                                        ScanOptions.ExtTrigger|ScanOptions.ConvertData
                                    ));
                

                int Count = 0;
                int Index = 0;
                short daqStatus;

                ushort[] theArray = new ushort[BUFFERSIZE];

                do
                {   //spin here until the buffer is half full
                    System.Threading.Thread.Sleep(10);
                    IsError(daq.GetStatus(out daqStatus, out Count, out Index, FunctionType.AiFunction));
                    if (Console.KeyAvailable)
                    {
                        IsError(daq.StopBackground(FunctionType.AiFunction));
                        MccService.WinBufFreeEx(buffer);
                        return;
                    }

                } while( Count < BUFFERSIZE );

                IsError(MccService.WinBufToArray(buffer, theArray, 0, BUFFERSIZE));

                DisplayData(theArray, BUFFERSIZE/CHANCOUNT);
                
                IsError(daq.StopBackground(FunctionType.AiFunction));

                MccService.WinBufFreeEx(buffer);

                WaitForKey();
            }
        }
        public static int GetBoardNum(string dev)
        {

            for (int BoardNum = 0; BoardNum < 99; BoardNum++)
            {
                MccDaq.MccBoard daq = new MccDaq.MccBoard(BoardNum);
                if (daq.BoardName.Contains(dev))
                {

                    Console.WriteLine("{0} board number = {1}\n", dev, BoardNum.ToString()); 
                    return BoardNum;
                }
            }
            return -1;
        }
        public static int IsError(ErrorInfo e)
        {
            if (e.Value != 0)
            {
                Console.WriteLine(e.Message);
                WaitForKey();
                return 1;
            }
            return 0;
        }
        public static void WaitForKey()
        {
            Console.WriteLine("\nPress <SpaceBar> to continue...");

            System.ConsoleKeyInfo cki;
            do
            {
                cki = Console.ReadKey();
            } while (cki.Key != ConsoleKey.Spacebar);
        }
        public static void DisplayData(ushort[] datArray, int rows)
        {

            int i = 0;
            float temp = 0;

            for (int row = 0; row < rows; row++)
            {

                for (int c = 0; c < CHANCOUNT; c++)
                {

                    IsError(daq.ToEngUnits(Range.Bip10Volts, datArray[i], out temp));
                    Console.Write("{0}\t", temp.ToString("0.000").PadLeft(10));
                    i++;
                }

                Console.Write("\r\n");
            }
        }
    }
}
